#include <stdio.h>
#include <stdlib.h>
#include <dlfcn.h>
#include <QApplication>
#include "pluginfactory.h"

#include <QtDesigner/QExtensionManager>

#include "iface/plugin1iface.h"

int main(int argc, char *argv[])
{

	QApplication app(argc,argv);

    void *plugin1_handle = dlopen("plugin1/libplugin1.so", RTLD_LAZY);
    if (!plugin1_handle)
        printf("plugin1 was not initialized: %s\n", dlerror());

    void *plugin2_handle = dlopen("plugin2/libplugin2.so", RTLD_LAZY);
    if (!plugin2_handle)
        printf("plugin2 was not initialized: %s\n", dlerror());

    void *plugin3_handle = dlopen("plugin3/libplugin3.so", RTLD_LAZY);
    if (!plugin3_handle)
        printf("plugin3 was not initialized: %s\n", dlerror());


    create_t* create_plugin1 = (create_t*)dlsym(plugin1_handle, "create");
    destroy_t* destroy_plugin1 = (destroy_t*)dlsym(plugin1_handle, "destroy");
    if (!create_plugin1 || !destroy_plugin1) {
        printf("can not load plugin1 symbols: %s\n", dlerror());
    }

    create_t* create_plugin2 = (create_t*)dlsym(plugin2_handle, "create");
    destroy_t* destroy_plugin2 = (destroy_t*)dlsym(plugin2_handle, "destroy");
    if (!create_plugin2 || !destroy_plugin2) {
        printf("can not load plugin2 symbols: %s\n", dlerror());
    }

    create_t* create_plugin3 = (create_t*)dlsym(plugin3_handle, "create");
    destroy_t* destroy_plugin3 = (destroy_t*)dlsym(plugin3_handle, "destroy");
    if (!create_plugin3 || !destroy_plugin3) {
        printf("can not load plugin3 symbols: %s\n", dlerror());
    }


    QExtensionManager *m = new QExtensionManager(0);

    AbstractPlugin *plugin1 = create_plugin1();
    AbstractPlugin *plugin2 = create_plugin2();
    AbstractPlugin *plugin3 = create_plugin3();

    plugin1->setExtensionManager(m);
    plugin2->setExtensionManager(m);
    plugin3->setExtensionManager(m);

    plugin2->process(plugin1);
    plugin2->process(plugin3);

    destroy_plugin1(plugin1);
    dlclose(plugin1_handle);
    destroy_plugin2(plugin2);
    dlclose(plugin2_handle);
    destroy_plugin3(plugin3);
    dlclose(plugin3_handle);

    return app.exec();
}
